/* 
 *  rocks/rstypes.h
 *
 *  Copyright (C) 2001 Victor Zandy
 *  See COPYING for distribution terms.
 */

#ifndef _RSTYPES_H_
#define _RSTYPES_H_

#include <netinet/ip.h>
#include <signal.h>
#include "ring.h"

#define MIN(x,y) ((x) < (y) ? (x) : (y))
#define MAX(x,y) ((x) > (y) ? (x) : (y))

#define RS_MAXFD 1024 /* FIXME: Use the system limit */

#ifndef NO_AUTH
/* Crypto implementation dependent declarations */
typedef struct rs_key *rs_key_t;
int rs_init_crypt();
void rs_key_free(rs_key_t key);
rs_key_t rs_key_exchange(int sock);
int rs_mutual_auth(rs_key_t key, int sock);

/* Common crypto functions */  
int rs_authenticate(rs_key_t key, int sock);
#else
typedef void *rs_key_t;
#endif /* NO_AUTH */

typedef struct hb * hb_t;

typedef enum { IOP_LISTEN, IOP_SERVER } iop_type_t;
typedef struct iop * iop_t;

typedef struct recstate recstate_t;
struct recstate {
	int sda, sdp;        /* active and passive socks */
	int sdn;             /* new incoming connection on sda */
	int sdc;             /* final connection */
	int sent;            /* Have we (if server) sent the byte? */
	int est;             /* Established connections bitset */
	int timeouts;	     /* Timeout counter  */
};

typedef enum { RS_NOTCONNECTED, RS_ESTABLISHED, RS_SUSPENDED } rs_state;
typedef enum { RS_ROLE_SERVER, RS_ROLE_CLIENT, RS_ROLE_LISTEN, RS_ROLE_UNDEF } rs_role;
extern char *rs_roles[]; /* If you add a role, update rs_roles. */

typedef struct rs_ * rs_t;
struct rs_ {
	rs_state state;
	int type;                   /* socket type */
	ring_t ring;                /* in flight buffer */
	unsigned rcvseq;            /* seq # of next byte to be passed to app */
	unsigned sndseq;            /* seq # of next byte to be passed to kernel */
	unsigned maxrcv;            /* TCP socket recv buffer size */
	unsigned maxsnd;            /* TCP socket send buffer size */
	int sd;                     /* socket descriptor */
	struct sockaddr_in sa_locl; /* current address of this endpoint */
	struct sockaddr_in sa_peer; /* current address of peer endpoint */
	hb_t hb;                    /* heartbeat state */
	rs_key_t key;               /* authentication key */
	recstate_t rec;             /* reconnection state */
	rs_t rec_next, rec_prev;    /* reconnection list */
	rs_role role;		    /* role in most recent connection */
	iop_t iop;           	    /* interoperability state */
	int booger;                 /* understand from the name that this is temporary */
};

/* util.h */
typedef struct rocklist *rocklist_t;
struct rocklist {
	rs_t rs;
	rocklist_t next;
};
int rs_rocklist_insert(rocklist_t *head, rs_t rs);
int rs_rocklist_remove(rocklist_t *head, rs_t rs);
rs_t rs_rocklist_findsa(rocklist_t head, struct sockaddr_in *sa);
void rs_rocklist_free(rocklist_t *head);


typedef enum { RS_MODE_RS, RS_MODE_NATIVE } rs_mode_t;
void rs_mode_push(rs_mode_t m);
void rs_mode_native();
void rs_mode_pop();

char *rs_ipstr(struct sockaddr_in *addr);

rs_t rs_lookup(int fd);
void rs_begin_reconnect(rs_t rs);
void rs_reconnect(rs_t rs);

int rs_init_connection(rs_t rs);

int rs_init_heartbeat();
void rs_stop_heartbeat(sigset_t *);
void rs_resume_heartbeat(sigset_t *);
hb_t rs_new_heartbeat();
void rs_free_heartbeat(hb_t hb);
int rs_hb_establish(int rock, hb_t hb, rs_role role);
int rs_hb_cancel(hb_t hb);

/* Fallback (interoperability) */
void rs_fallback(rs_t rs);

/* Interoperability */
void rs_free_iop(rs_t rs);
int rs_iop_listener(rs_t rs);
int rs_iop_accept(rs_t listener, rs_t rs);
int rs_iop_connect(rs_t rs);
int rs_iop_eof(rs_t rs);
int rs_iop_isdontclose(rs_t rs);

#endif
